@@ -12,8 +12,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tcl.h,v 1.254.2.13 2009/11/03 19:21:37 dgp Exp $
  */
 
 #ifndef _TCL
@@ -60,10 +58,10 @@ extern "C" {
 #define TCL_MAJOR_VERSION   8
 #define TCL_MINOR_VERSION   5
 #define TCL_RELEASE_LEVEL   TCL_FINAL_RELEASE
-#define TCL_RELEASE_SERIAL  8
+#define TCL_RELEASE_SERIAL  19
 
 #define TCL_VERSION	    "8.5"
-#define TCL_PATCH_LEVEL	    "8.5.8"
+#define TCL_PATCH_LEVEL	    "8.5.19"
 
 /*
  * The following definitions set up the proper options for Windows compilers.
@@ -71,7 +69,7 @@ extern "C" {
  */
 
 #ifndef __WIN32__
-#   if defined(_WIN32) || defined(WIN32) || defined(__MINGW32__) || defined(__BORLANDC__) || (defined(__WATCOMC__) && defined(__WINDOWS_386__))
+#   if defined(_WIN32) || defined(WIN32) || defined(__MSVCRT__) || defined(__BORLANDC__) || (defined(__WATCOMC__) && defined(__WINDOWS_386__))
 #	define __WIN32__
 #	ifndef WIN32
 #	    define WIN32
@@ -145,7 +143,7 @@ extern "C" {
  *
  * The following TCL_VARARGS* macros are to support old extensions
  * written for older versions of Tcl where the macros permitted
- * support for the varargs.h system as well as stdarg.h .  
+ * support for the varargs.h system as well as stdarg.h .
  *
  * New code should just directly be written to use stdarg.h conventions.
  */
@@ -167,10 +165,10 @@ extern "C" {
  * Note: when building static but linking dynamically to MSVCRT we must still
  *       correctly decorate the C library imported function.  Use CRTIMPORT
  *       for this purpose.  _DLL is defined by the compiler when linking to
- *       MSVCRT.  
+ *       MSVCRT.
  */
 
-#if (defined(__WIN32__) && (defined(_MSC_VER) || (__BORLANDC__ >= 0x0550) || defined(__LCC__) || defined(__WATCOMC__) || (defined(__GNUC__) && defined(__declspec))))
+#if (defined(__WIN32__) && (defined(_MSC_VER) || (defined(__BORLANDC__) && (__BORLANDC__ >= 0x0550)) || defined(__LCC__) || defined(__WATCOMC__) || (defined(__GNUC__) && defined(__declspec))))
 #   define HAVE_DECLSPEC 1
 #   ifdef STATIC_BUILD
 #       define DLLIMPORT
@@ -262,7 +260,7 @@ extern "C" {
 #endif
 
 /*
- * Make sure EXTERN isn't defined elsewhere
+ * Make sure EXTERN isn't defined elsewhere.
  */
 
 #ifdef EXTERN
@@ -295,10 +293,12 @@ typedef long LONG;
  * in ANSI C; maps them to type "char *" in non-ANSI systems.
  */
 
-#ifndef NO_VOID
-#define VOID	void
-#else
-#define VOID	char
+#ifndef __VXWORKS__
+#   ifndef NO_VOID
+#	define VOID void
+#   else
+#	define VOID char
+#   endif
 #endif
 
 /*
@@ -315,7 +315,7 @@ typedef long LONG;
 #endif
 
 /*
- * Darwin specifc configure overrides (to support fat compiles, where
+ * Darwin specific configure overrides (to support fat compiles, where
  * configure runs only once for multiple architectures):
  */
 
@@ -354,28 +354,17 @@ typedef long LONG;
  */
 
 #if !defined(TCL_WIDE_INT_TYPE)&&!defined(TCL_WIDE_INT_IS_LONG)
-#   if defined(__GNUC__)
-#      define TCL_WIDE_INT_TYPE long long
-#      if defined(__WIN32__) && !defined(__CYGWIN__)
-#         define TCL_LL_MODIFIER        "I64"
-#      else
-#         define TCL_LL_MODIFIER	"L"
-#      endif
-typedef struct stat	Tcl_StatBuf;
-#   elif defined(__WIN32__)
+#   if defined(__WIN32__)
 #      define TCL_WIDE_INT_TYPE __int64
 #      ifdef __BORLANDC__
-typedef struct stati64 Tcl_StatBuf;
 #         define TCL_LL_MODIFIER	"L"
 #      else /* __BORLANDC__ */
-#         if _MSC_VER < 1400 || !defined(_M_IX86)
-typedef struct _stati64	Tcl_StatBuf;
-#         else
-typedef struct _stat64	Tcl_StatBuf;
-#         endif /* _MSC_VER < 1400 */
 #         define TCL_LL_MODIFIER	"I64"
 #      endif /* __BORLANDC__ */
-#   else /* __WIN32__ */
+#   elif defined(__GNUC__)
+#      define TCL_WIDE_INT_TYPE long long
+#      define TCL_LL_MODIFIER	"ll"
+#   else /* ! __WIN32__ && ! __GNUC__ */
 /*
  * Don't know what platform it is and configure hasn't discovered what is
  * going on for us. Try to guess...
@@ -401,7 +390,6 @@ typedef TCL_WIDE_INT_TYPE		Tcl_WideInt;
 typedef unsigned TCL_WIDE_INT_TYPE	Tcl_WideUInt;
 
 #ifdef TCL_WIDE_INT_IS_LONG
-typedef struct stat	Tcl_StatBuf;
 #   define Tcl_WideAsLong(val)		((long)(val))
 #   define Tcl_LongAsWide(val)		((long)(val))
 #   define Tcl_WideAsDouble(val)	((double)((long)(val)))
@@ -415,11 +403,6 @@ typedef struct stat	Tcl_StatBuf;
  * or some other strange platform.
  */
 #   ifndef TCL_LL_MODIFIER
-#      ifdef HAVE_STRUCT_STAT64
-typedef struct stat64	Tcl_StatBuf;
-#      else
-typedef struct stat	Tcl_StatBuf;
-#      endif /* HAVE_STRUCT_STAT64 */
 #      define TCL_LL_MODIFIER		"ll"
 #   endif /* !TCL_LL_MODIFIER */
 #   define Tcl_WideAsLong(val)		((long)((Tcl_WideInt)(val)))
@@ -428,6 +411,39 @@ typedef struct stat	Tcl_StatBuf;
 #   define Tcl_DoubleAsWide(val)	((Tcl_WideInt)((double)(val)))
 #endif /* TCL_WIDE_INT_IS_LONG */
 
+#if defined(__WIN32__)
+#   ifdef __BORLANDC__
+	typedef struct stati64 Tcl_StatBuf;
+#   elif defined(_WIN64)
+	typedef struct __stat64 Tcl_StatBuf;
+#   elif (defined(_MSC_VER) && (_MSC_VER < 1400)) || defined(_USE_32BIT_TIME_T)
+	typedef struct _stati64	Tcl_StatBuf;
+#   else
+	typedef struct _stat32i64 Tcl_StatBuf;
+#   endif /* _MSC_VER < 1400 */
+#elif defined(__CYGWIN__)
+    typedef struct {
+	dev_t st_dev;
+	unsigned short st_ino;
+	unsigned short st_mode;
+	short st_nlink;
+	short st_uid;
+	short st_gid;
+	/* Here is a 2-byte gap */
+	dev_t st_rdev;
+	/* Here is a 4-byte gap */
+	long long st_size;
+	struct {long tv_sec;} st_atim;
+	struct {long tv_sec;} st_mtim;
+	struct {long tv_sec;} st_ctim;
+	/* Here is a 4-byte gap */
+    } Tcl_StatBuf;
+#elif defined(HAVE_STRUCT_STAT64) && !defined(__APPLE__)
+    typedef struct stat64 Tcl_StatBuf;
+#else
+    typedef struct stat Tcl_StatBuf;
+#endif
+
 /*
  * Data structures defined opaquely in this module. The definitions below just
  * provide dummy types. A few fields are made visible in Tcl_Interp
@@ -514,9 +530,10 @@ typedef void (Tcl_ThreadCreateProc) _ANSI_ARGS_((ClientData clientData));
  * given to Tcl_CreateThread.
  */
 
-#define TCL_THREAD_STACK_DEFAULT (0)    /* Use default size for stack */
-#define TCL_THREAD_NOFLAGS	 (0000) /* Standard flags, default behaviour */
-#define TCL_THREAD_JOINABLE	 (0001) /* Mark the thread as joinable */
+#define TCL_THREAD_STACK_DEFAULT (0)    /* Use default size for stack. */
+#define TCL_THREAD_NOFLAGS	 (0000) /* Standard flags, default
+					 * behaviour. */
+#define TCL_THREAD_JOINABLE	 (0001) /* Mark the thread as joinable. */
 
 /*
  * Flag values passed to Tcl_StringCaseMatch.
@@ -528,20 +545,20 @@ typedef void (Tcl_ThreadCreateProc) _ANSI_ARGS_((ClientData clientData));
  * Flag values passed to Tcl_GetRegExpFromObj.
  */
 
-#define	TCL_REG_BASIC		000000	/* BREs (convenience) */
-#define	TCL_REG_EXTENDED	000001	/* EREs */
-#define	TCL_REG_ADVF		000002	/* advanced features in EREs */
-#define	TCL_REG_ADVANCED	000003	/* AREs (which are also EREs) */
-#define	TCL_REG_QUOTE		000004	/* no special characters, none */
-#define	TCL_REG_NOCASE		000010	/* ignore case */
-#define	TCL_REG_NOSUB		000020	/* don't care about subexpressions */
-#define	TCL_REG_EXPANDED	000040	/* expanded format, white space &
-					 * comments */
+#define	TCL_REG_BASIC		000000	/* BREs (convenience). */
+#define	TCL_REG_EXTENDED	000001	/* EREs. */
+#define	TCL_REG_ADVF		000002	/* Advanced features in EREs. */
+#define	TCL_REG_ADVANCED	000003	/* AREs (which are also EREs). */
+#define	TCL_REG_QUOTE		000004	/* No special characters, none. */
+#define	TCL_REG_NOCASE		000010	/* Ignore case. */
+#define	TCL_REG_NOSUB		000020	/* Don't care about subexpressions. */
+#define	TCL_REG_EXPANDED	000040	/* Expanded format, white space &
+					 * comments. */
 #define	TCL_REG_NLSTOP		000100  /* \n doesn't match . or [^ ] */
-#define	TCL_REG_NLANCH		000200  /* ^ matches after \n, $ before */
-#define	TCL_REG_NEWLINE		000300  /* newlines are line terminators */
-#define	TCL_REG_CANMATCH	001000  /* report details on partial/limited
-					 * matches */
+#define	TCL_REG_NLANCH		000200  /* ^ matches after \n, $ before. */
+#define	TCL_REG_NEWLINE		000300  /* Newlines are line terminators. */
+#define	TCL_REG_CANMATCH	001000  /* Report details on partial/limited
+					 * matches. */
 
 /*
  * Flags values passed to Tcl_RegExpExecObj.
@@ -600,13 +617,13 @@ typedef struct stat *Tcl_OldStat_;
  *			interpreter's result is meaningless.
  */
 
-#define TCL_OK		0
-#define TCL_ERROR	1
-#define TCL_RETURN	2
-#define TCL_BREAK	3
-#define TCL_CONTINUE	4
+#define TCL_OK			0
+#define TCL_ERROR		1
+#define TCL_RETURN		2
+#define TCL_BREAK		3
+#define TCL_CONTINUE		4
 
-#define TCL_RESULT_SIZE 200
+#define TCL_RESULT_SIZE		200
 
 /*
  * Flags to control what substitutions are performed by Tcl_SubstObj():
@@ -761,19 +778,19 @@ typedef struct Tcl_Obj {
 				 * internal rep. NULL indicates the object has
 				 * no internal rep (has no type). */
     union {			/* The internal representation: */
-	long longValue;		/*   - an long integer value */
-	double doubleValue;	/*   - a double-precision floating value */
-	VOID *otherValuePtr;	/*   - another, type-specific value */
-	Tcl_WideInt wideValue;	/*   - a long long value */
-	struct {		/*   - internal rep as two pointers */
+	long longValue;		/*   - an long integer value. */
+	double doubleValue;	/*   - a double-precision floating value. */
+	VOID *otherValuePtr;	/*   - another, type-specific value. */
+	Tcl_WideInt wideValue;	/*   - a long long value. */
+	struct {		/*   - internal rep as two pointers. */
 	    VOID *ptr1;
 	    VOID *ptr2;
 	} twoPtrValue;
 	struct {		/*   - internal rep as a wide int, tightly
-				 *     packed fields */
-	    VOID *ptr;		/* Pointer to digits */
+				 *     packed fields. */
+	    VOID *ptr;		/* Pointer to digits. */
 	    unsigned long value;/* Alloc, used, and signum packed into a
-				 * single word */
+				 * single word. */
 	} ptrAndLongRep;
     } internalRep;
 } Tcl_Obj;
@@ -783,10 +800,7 @@ typedef struct Tcl_Obj {
  * whether an object is shared (i.e. has reference count > 1). Note: clients
  * should use Tcl_DecrRefCount() when they are finished using an object, and
  * should never call TclFreeObj() directly. TclFreeObj() is only defined and
- * made public in tcl.h to support Tcl_DecrRefCount's macro definition. Note
- * also that Tcl_DecrRefCount() refers to the parameter "obj" twice. This
- * means that you should avoid calling it with an expression that is expensive
- * to compute or has side effects.
+ * made public in tcl.h to support Tcl_DecrRefCount's macro definition.
  */
 
 void		Tcl_IncrRefCount _ANSI_ARGS_((Tcl_Obj *objPtr));
@@ -859,16 +873,17 @@ typedef struct Tcl_CallFrame {
     Tcl_Namespace *nsPtr;
     int dummy1;
     int dummy2;
-    char *dummy3;
-    char *dummy4;
-    char *dummy5;
+    VOID *dummy3;
+    VOID *dummy4;
+    VOID *dummy5;
     int dummy6;
-    char *dummy7;
-    char *dummy8;
+    VOID *dummy7;
+    VOID *dummy8;
     int dummy9;
-    char *dummy10;
-    char *dummy11;
-    char *dummy12;
+    VOID *dummy10;
+    VOID *dummy11;
+    VOID *dummy12;
+    VOID *dummy13;
 } Tcl_CallFrame;
 
 /*
@@ -935,8 +950,8 @@ typedef struct Tcl_DString {
  * buffer space required by Tcl_PrintDouble.
  */
 
-#define TCL_MAX_PREC 17
-#define TCL_DOUBLE_SPACE (TCL_MAX_PREC+10)
+#define TCL_MAX_PREC		17
+#define TCL_DOUBLE_SPACE	(TCL_MAX_PREC+10)
 
 /*
  * Definition for a number of bytes of buffer space sufficient to hold the
@@ -954,8 +969,6 @@ typedef struct Tcl_DString {
  *	is safe to leave the hash unquoted when the element is not the first
  *	element of a list, and this flag can be used by the caller to indicate
  *	that condition.
- * (Careful! If you change these flag values be sure to change the definitions
- * at the front of tclUtil.c).
  */
 
 #define TCL_DONT_USE_BRACES	1
@@ -993,9 +1006,9 @@ typedef struct Tcl_DString {
  * page for details):
  */
 
-#define TCL_VOLATILE	((Tcl_FreeProc *) 1)
-#define TCL_STATIC	((Tcl_FreeProc *) 0)
-#define TCL_DYNAMIC	((Tcl_FreeProc *) 3)
+#define TCL_VOLATILE		((Tcl_FreeProc *) 1)
+#define TCL_STATIC		((Tcl_FreeProc *) 0)
+#define TCL_DYNAMIC		((Tcl_FreeProc *) 3)
 
 /*
  * Flag values passed to variable-related functions.
@@ -1348,9 +1361,9 @@ typedef void (Tcl_ScaleTimeProc) _ANSI_ARGS_((Tcl_Time *timebuf,
  * indicate what sorts of events are of interest:
  */
 
-#define TCL_READABLE	(1<<1)
-#define TCL_WRITABLE	(1<<2)
-#define TCL_EXCEPTION	(1<<3)
+#define TCL_READABLE		(1<<1)
+#define TCL_WRITABLE		(1<<2)
+#define TCL_EXCEPTION		(1<<3)
 
 /*
  * Flag values to pass to Tcl_OpenCommandChannel to indicate the disposition
@@ -1368,15 +1381,15 @@ typedef void (Tcl_ScaleTimeProc) _ANSI_ARGS_((Tcl_Time *timebuf,
  * should be closed.
  */
 
-#define TCL_CLOSE_READ	(1<<1)
-#define TCL_CLOSE_WRITE	(1<<2)
+#define TCL_CLOSE_READ		(1<<1)
+#define TCL_CLOSE_WRITE		(1<<2)
 
 /*
  * Value to use as the closeProc for a channel that supports the close2Proc
  * interface.
  */
 
-#define TCL_CLOSE2PROC	((Tcl_DriverCloseProc *)1)
+#define TCL_CLOSE2PROC		((Tcl_DriverCloseProc *) 1)
 
 /*
  * Channel version tag. This was introduced in 8.3.2/8.4.
@@ -1389,7 +1402,7 @@ typedef void (Tcl_ScaleTimeProc) _ANSI_ARGS_((Tcl_Time *timebuf,
 #define TCL_CHANNEL_VERSION_5	((Tcl_ChannelTypeVersion) 0x5)
 
 /*
- * TIP #218: Channel Actions, Ids for Tcl_DriverThreadActionProc
+ * TIP #218: Channel Actions, Ids for Tcl_DriverThreadActionProc.
  */
 
 #define TCL_CHANNEL_THREAD_INSERT (0)
@@ -1485,7 +1498,7 @@ typedef struct Tcl_ChannelType {
 				/* Set blocking mode for the raw channel. May
 				 * be NULL. */
     /*
-     * Only valid in TCL_CHANNEL_VERSION_2 channels or later
+     * Only valid in TCL_CHANNEL_VERSION_2 channels or later.
      */
     Tcl_DriverFlushProc *flushProc;
 				/* Function to call to flush a channel. May be
@@ -1495,7 +1508,7 @@ typedef struct Tcl_ChannelType {
 				 * This will be passed up the stacked channel
 				 * chain. */
     /*
-     * Only valid in TCL_CHANNEL_VERSION_3 channels or later
+     * Only valid in TCL_CHANNEL_VERSION_3 channels or later.
      */
     Tcl_DriverWideSeekProc *wideSeekProc;
 				/* Function to call to seek on the channel
@@ -1503,8 +1516,8 @@ typedef struct Tcl_ChannelType {
 				 * NULL, and must be NULL if seekProc is
 				 * NULL. */
     /*
-     * Only valid in TCL_CHANNEL_VERSION_4 channels or later
-     * TIP #218, Channel Thread Actions
+     * Only valid in TCL_CHANNEL_VERSION_4 channels or later.
+     * TIP #218, Channel Thread Actions.
      */
     Tcl_DriverThreadActionProc *threadActionProc;
 				/* Function to call to notify the driver of
@@ -1512,8 +1525,8 @@ typedef struct Tcl_ChannelType {
 				 * be NULL. */
 
     /*
-     * Only valid in TCL_CHANNEL_VERSION_5 channels or later
-     * TIP #208, File Truncation
+     * Only valid in TCL_CHANNEL_VERSION_5 channels or later.
+     * TIP #208, File Truncation.
      */
     Tcl_DriverTruncateProc *truncateProc;
 				/* Function to call to truncate the underlying
@@ -1547,14 +1560,14 @@ typedef enum Tcl_PathType {
  */
 
 typedef struct Tcl_GlobTypeData {
-    int type;			/* Corresponds to bcdpfls as in 'find -t' */
-    int perm;			/* Corresponds to file permissions */
-    Tcl_Obj *macType;		/* Acceptable mac type */
-    Tcl_Obj *macCreator;	/* Acceptable mac creator */
+    int type;			/* Corresponds to bcdpfls as in 'find -t'. */
+    int perm;			/* Corresponds to file permissions. */
+    Tcl_Obj *macType;		/* Acceptable Mac type. */
+    Tcl_Obj *macCreator;	/* Acceptable Mac creator. */
 } Tcl_GlobTypeData;
 
 /*
- * Type and permission definitions for glob command
+ * Type and permission definitions for glob command.
  */
 
 #define TCL_GLOB_TYPE_BLOCK		(1<<0)
@@ -1573,7 +1586,7 @@ typedef struct Tcl_GlobTypeData {
 #define TCL_GLOB_PERM_X			(1<<4)
 
 /*
- * Flags for the unload callback function
+ * Flags for the unload callback function.
  */
 
 #define TCL_UNLOAD_DETACH_FROM_INTERPRETER	(1<<0)
@@ -2135,7 +2148,7 @@ typedef struct Tcl_Parse {
  * reflected in regcustom.h.
  */
 
-#if TCL_UTF_MAX > 3
+#if TCL_UTF_MAX > 4
     /*
      * unsigned int isn't 100% accurate as it should be a strict 4-byte value
      * (perhaps wchar_t). 64-bit systems may have troubles. The size of this
@@ -2157,9 +2170,9 @@ typedef unsigned short Tcl_UniChar;
 
 typedef struct Tcl_Config {
     CONST char *key;		/* Configuration key to register. ASCII
-				 * encoded, thus UTF-8 */
+				 * encoded, thus UTF-8. */
     CONST char *value;		/* The value associated with the key. System
-				 * encoding */
+				 * encoding. */
 } Tcl_Config;
 
 /*
@@ -2192,7 +2205,7 @@ typedef unsigned int mp_digit;
  * value since the stubs tables don't match.
  */
 
-#define TCL_STUB_MAGIC ((int)0xFCA3BACF)
+#define TCL_STUB_MAGIC		((int) 0xFCA3BACF)
 
 /*
  * The following function is required to be defined in all stubs aware
@@ -2297,11 +2310,16 @@ EXTERN void		Tcl_GetMemoryInfo _ANSI_ARGS_((Tcl_DString *dsPtr));
 #   define Tcl_IncrRefCount(objPtr) \
 	++(objPtr)->refCount
     /*
-     * Use do/while0 idiom for optimum correctness without compiler warnings
+     * Use do/while0 idiom for optimum correctness without compiler warnings.
      * http://c2.com/cgi/wiki?TrivialDoWhileLoop
      */
 #   define Tcl_DecrRefCount(objPtr) \
-	do { if (--(objPtr)->refCount <= 0) TclFreeObj(objPtr); } while(0)
+	do { \
+	    Tcl_Obj *_objPtr = (objPtr); \
+	    if (--(_objPtr)->refCount <= 0) { \
+		TclFreeObj(_objPtr); \
+	    } \
+	} while(0)
 #   define Tcl_IsShared(objPtr) \
 	((objPtr)->refCount > 1)
 #endif
@@ -2390,17 +2408,6 @@ EXTERN void		Tcl_GetMemoryInfo _ANSI_ARGS_((Tcl_DString *dsPtr));
 #endif /* TCL_THREADS */
 
 #ifndef TCL_NO_DEPRECATED
-    /*
-     * Deprecated Tcl functions:
-     */
-
-#   undef  Tcl_EvalObj
-#   define Tcl_EvalObj(interp,objPtr) \
-	Tcl_EvalObjEx((interp),(objPtr),0)
-#   undef  Tcl_GlobalEvalObj
-#   define Tcl_GlobalEvalObj(interp,objPtr) \
-	Tcl_EvalObjEx((interp),(objPtr),TCL_EVAL_GLOBAL)
-
     /*
      * These function have been renamed. The old names are deprecated, but we
      * define these macros for backwards compatibilty.
