@@ -11,8 +11,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tclMacOSXNotify.c,v 1.18.2.5 2009/08/24 00:27:53 das Exp $
  */
 
 #include "tclInt.h"
@@ -1379,7 +1377,8 @@ UpdateWaitingListAndServiceEvents(
 	if (tsdPtr->runLoopTimer && !tsdPtr->runLoopServicingEvents &&
 		(tsdPtr->runLoopNestingLevel > 1 || !tsdPtr->runLoopRunning)) {
 	    tsdPtr->runLoopServicingEvents = 1;
-	    while (Tcl_ServiceAll() && tsdPtr->waitTime == 0) {}
+           /* This call seems to simply force event processing through and prevents hangups that have long been observed with Tk-Cocoa.  */
+	    Tcl_ServiceAll();
 	    tsdPtr->runLoopServicingEvents = 0;
 	}
 	break;
