@@ -28,9 +28,6 @@
 # Copyright (c) 2004 by Kevin B. Kenny.	 All rights reserved.
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-#
-# RCS: @(#) $Id: tclZIC.tcl,v 1.9.4.1 2009/04/09 20:12:10 kennykb Exp $
-#
 #----------------------------------------------------------------------
 
 package require Tcl 8.5
@@ -397,6 +394,9 @@ proc parseON {on} {
 #----------------------------------------------------------------------
 
 proc onDayOfMonth {day year month} {
+    scan $day %d day
+    scan $year %d year
+    scan $month %d month
     set date [::tcl::clock::GetJulianDayFromEraYearMonthDay \
 	    [dict create era CE year $year month $month dayOfMonth $day] \
 		 2361222]
