@@ -17,10 +17,10 @@
  * Prototypes for procedures defined later in this file:
  */
 
-static int    Pkgb_SubObjCmd _ANSI_ARGS_((ClientData clientData,
-		Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
-static int    Pkgb_UnsafeObjCmd _ANSI_ARGS_((ClientData clientData,
-		Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
+static int    Pkgb_SubObjCmd(ClientData clientData,
+		Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
+static int    Pkgb_UnsafeObjCmd(ClientData clientData,
+		Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
 
 /*
  *----------------------------------------------------------------------
@@ -44,11 +44,11 @@ static int    Pkgb_UnsafeObjCmd _ANSI_ARGS_((ClientData clientData,
 #endif
 
 static int
-Pkgb_SubObjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
-    Tcl_Interp *interp;		/* Current interpreter. */
-    int objc;			/* Number of arguments. */
-    Tcl_Obj *CONST objv[];	/* Argument objects. */
+Pkgb_SubObjCmd(
+    ClientData dummy,		/* Not used. */
+    Tcl_Interp *interp,		/* Current interpreter. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *CONST objv[])	/* Argument objects. */
 {
     int first, second;
 
@@ -85,11 +85,11 @@ Pkgb_SubObjCmd(dummy, interp, objc, objv)
  */
 
 static int
-Pkgb_UnsafeObjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
-    Tcl_Interp *interp;		/* Current interpreter. */
-    int objc;			/* Number of arguments. */
-    Tcl_Obj *CONST objv[];	/* Argument objects. */
+Pkgb_UnsafeObjCmd(
+    ClientData dummy,		/* Not used. */
+    Tcl_Interp *interp,		/* Current interpreter. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *CONST objv[])	/* Argument objects. */
 {
     return Tcl_EvalEx(interp, "list unsafe command invoked", -1, TCL_EVAL_GLOBAL);
 }
@@ -112,8 +112,8 @@ Pkgb_UnsafeObjCmd(dummy, interp, objc, objv)
  */
 
 DLLEXPORT int
-Pkgb_Init(interp)
-    Tcl_Interp *interp;		/* Interpreter in which the package is to be
+Pkgb_Init(
+    Tcl_Interp *interp)		/* Interpreter in which the package is to be
 				 * made available. */
 {
     int code;
@@ -153,8 +153,8 @@ Pkgb_Init(interp)
  */
 
 DLLEXPORT int
-Pkgb_SafeInit(interp)
-    Tcl_Interp *interp;		/* Interpreter in which the package is to be
+Pkgb_SafeInit(
+    Tcl_Interp *interp)		/* Interpreter in which the package is to be
 				 * made available. */
 {
     int code;
