@@ -1,33 +1,34 @@
-/* 
+/*
  * pkgc.c --
  *
- *	This file contains a simple Tcl package "pkgc" that is intended
- *	for testing the Tcl dynamic loading facilities.  It can be used
- *	in both safe and unsafe interpreters.
+ *	This file contains a simple Tcl package "pkgc" that is intended for
+ *	testing the Tcl dynamic loading facilities. It can be used in both
+ *	safe and unsafe interpreters.
  *
  * Copyright (c) 1995 Sun Microsystems, Inc.
  *
- * See the file "license.terms" for information on usage and redistribution
- * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ * See the file "license.terms" for information on usage and redistribution of
+ * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
+
 #include "tcl.h"
 
 /*
  * Prototypes for procedures defined later in this file:
  */
 
-static int    Pkgc_SubObjCmd _ANSI_ARGS_((ClientData clientData,
-		Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[]));
-static int    Pkgc_UnsafeObjCmd _ANSI_ARGS_((ClientData clientData,
-		Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[]));
+static int    Pkgc_SubObjCmd(ClientData clientData,
+		Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
+static int    Pkgc_UnsafeObjCmd(ClientData clientData,
+		Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
 
 /*
  *----------------------------------------------------------------------
  *
  * Pkgc_SubObjCmd --
  *
- *	This procedure is invoked to process the "pkgc_sub" Tcl command.
- *	It expects two arguments and returns their difference.
+ *	This procedure is invoked to process the "pkgc_sub" Tcl command. It
+ *	expects two arguments and returns their difference.
  *
  * Results:
  *	A standard Tcl result.
@@ -39,11 +40,11 @@ static int    Pkgc_UnsafeObjCmd _ANSI_ARGS_((ClientData clientData,
  */
 
 static int
-Pkgc_SubObjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
-    Tcl_Interp *interp;		/* Current interpreter. */
-    int objc;			/* Number of arguments. */
-    Tcl_Obj * CONST objv[];	/* Argument objects. */
+Pkgc_SubObjCmd(
+    ClientData dummy,		/* Not used. */
+    Tcl_Interp *interp,		/* Current interpreter. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *CONST objv[])	/* Argument objects. */
 {
     int first, second;
 
@@ -64,8 +65,8 @@ Pkgc_SubObjCmd(dummy, interp, objc, objv)
  *
  * Pkgc_UnsafeCmd --
  *
- *	This procedure is invoked to process the "pkgc_unsafe" Tcl command.
- *	It just returns a constant string.
+ *	This procedure is invoked to process the "pkgc_unsafe" Tcl command. It
+ *	just returns a constant string.
  *
  * Results:
  *	A standard Tcl result.
@@ -77,11 +78,11 @@ Pkgc_SubObjCmd(dummy, interp, objc, objv)
  */
 
 static int
-Pkgc_UnsafeObjCmd(dummy, interp, objc, objv)
-    ClientData dummy;		/* Not used. */
-    Tcl_Interp *interp;		/* Current interpreter. */
-    int objc;			/* Number of arguments. */
-    Tcl_Obj * CONST objv[];	/* Argument objects. */
+Pkgc_UnsafeObjCmd(
+    ClientData dummy,		/* Not used. */
+    Tcl_Interp *interp,		/* Current interpreter. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *CONST objv[])	/* Argument objects. */
 {
     Tcl_SetObjResult(interp, Tcl_NewStringObj("unsafe command invoked", -1));
     return TCL_OK;
@@ -92,8 +93,8 @@ Pkgc_UnsafeObjCmd(dummy, interp, objc, objv)
  *
  * Pkgc_Init --
  *
- *	This is a package initialization procedure, which is called
- *	by Tcl when this package is to be added to an interpreter.
+ *	This is a package initialization procedure, which is called by Tcl
+ *	when this package is to be added to an interpreter.
  *
  * Results:
  *	None.
@@ -105,13 +106,13 @@ Pkgc_UnsafeObjCmd(dummy, interp, objc, objv)
  */
 
 int
-Pkgc_Init(interp)
-    Tcl_Interp *interp;		/* Interpreter in which the package is
-				 * to be made available. */
+Pkgc_Init(
+    Tcl_Interp *interp)		/* Interpreter in which the package is to be
+				 * made available. */
 {
     int code;
 
-    if (Tcl_InitStubs(interp, TCL_VERSION, 1) == NULL) {
+    if (Tcl_InitStubs(interp, TCL_VERSION, 0) == NULL) {
 	return TCL_ERROR;
     }
     code = Tcl_PkgProvide(interp, "Pkgc", "1.7.2");
@@ -130,8 +131,8 @@ Pkgc_Init(interp)
  *
  * Pkgc_SafeInit --
  *
- *	This is a package initialization procedure, which is called
- *	by Tcl when this package is to be added to an unsafe interpreter.
+ *	This is a package initialization procedure, which is called by Tcl
+ *	when this package is to be added to a safe interpreter.
  *
  * Results:
  *	None.
@@ -143,18 +144,18 @@ Pkgc_Init(interp)
  */
 
 int
-Pkgc_SafeInit(interp)
-    Tcl_Interp *interp;		/* Interpreter in which the package is
-				 * to be made available. */
+Pkgc_SafeInit(
+    Tcl_Interp *interp)		/* Interpreter in which the package is to be
+				 * made available. */
 {
     int code;
 
-    if (Tcl_InitStubs(interp, TCL_VERSION, 1) == NULL) {
+    if (Tcl_InitStubs(interp, TCL_VERSION, 0) == NULL) {
 	return TCL_ERROR;
     }
     code = Tcl_PkgProvide(interp, "Pkgc", "1.7.2");
     if (code != TCL_OK) {
-      return code;
+	return code;
     }
     Tcl_CreateObjCommand(interp, "pkgc_sub", Pkgc_SubObjCmd, (ClientData) 0,
 	    (Tcl_CmdDeleteProc *) NULL);
